@@ -1,3 +1,6 @@
+/* Define to 1 if have PostgreSQL header files.  */
+#undef HAVE_LIBPQ_FE_H
+
 /* Define to 1 if sys/sem.h defines `union semun' type.  */
 #undef HAVE_SEMUN
 
