@@ -41,5 +41,7 @@
 #define HAVE_SYS_MMAN_H 1
 #define HAVE_MNTENT_H 1
 #define HAVE_ASSERT_H 1
+#define HAVE_POLL_H 1
 
 #define LLSEEK_OFFSET_T loff_t
+#define FUNC_SYNC_VOID 1
