@@ -1,4 +1,5 @@
-#if 1
+#if USE_INTERNAL_IPC_SYSCALL
+/* This version may work with some versions of libc/kernel.  */
 #define __KERNEL__
 #include <linux/ipc.h>
 #ifndef LINUX_GLIBC1
@@ -13,16 +14,19 @@ int chkr_shmctl (int shmid, int cmd, struct shmid_ds *buf);
 int
 chkr_shmctl (int shmid, int cmd, struct shmid_ds *buf)
 {
-    return chkr__ipc (SHMCTL, shmid, cmd, 0, buf);
+  return chkr__ipc (SHMCTL, shmid, cmd, 0, buf);
 }
-#else
 
+#else /* USE_INTERNAL_IPC_SYSCALL  */
+
+/* This version should work with any libc configuration.  */
+/* However, it requires system call not to be hooked.  */
 #include <sys/ipc.h>
 #include <sys/shm.h>
 
 int
 chkr_shmctl (int shmid, int cmd, struct shmid_ds *buf)
 {
-  return shmctl(shmid, cmd, buf);
+  return shmctl (shmid, cmd, buf);
 }
-#endif
+#endif /* !USE_INTERNAL_IPC_SYSCALL  */
