@@ -59,11 +59,14 @@ static void search_in_block (struct malloc_header *block, int sure);
 /* Check an address
    Does a recursive descent if on heap.
    If SURE is true, the address comes not from a POINT_MAYBE block.
+   This function is not statis, because it can be called from 
+   search_in_registers.
 */
 void
 chkr_address (PTR ptr, int sure)
 {
   struct malloc_header *block;
+  int ptr_is_sure;
 #if 1
   struct mdesc *mdp;
   unsigned int first, last, middle;
@@ -112,26 +115,48 @@ chkr_address (PTR ptr, int sure)
   if (block->state != MDBUSY)
     return;		/* Pointer on free (or aged) block.  */
 
+  ptr_is_sure = sure && ptr == header_to_block (block);
+
   /* Can't change the status when the block is already pointed.
      Progression (POINT_MAYBE -> POINT_SURE) if SURE is true.  */
-  if ( ((block->garbage_t & POINT_MASK) != POINT_NOT)
-       && !((block->garbage_t & POINT_MASK) == POINT_MAYBE && sure))
-    return;
+  switch (block->garbage_t & POINT_MASK)
+    {
+    case POINT_NOT:
+      break;
+    case POINT_SURE:
+      return;
+    case POINT_MAYBE:
+      if (!sure)
+	return;
+      else
+	break;
+    case POINT_SEEN:
+      /* A leak is becoming a non-leak !  */
+      if (ptr_is_sure)
+	chkr_printf ("A leak was recupered!\n");
+      else
+	return;
+      break;
+    case POINT_LEAK:
+      /* Cannot happen due to initialisation.  */
+      chkr_abort ();
+    default:
+      chkr_abort ();
+    }
     
   /* if ptr == real_ptr + (...), ptr points on the begin of the block.  */
-  if (sure && ptr == header_to_block (block))
+  if (ptr_is_sure)
     block->garbage_t = POINT_SURE | (block->garbage_t & INUSE_MASK);
   else
     {
-      /* If the block was already a potential leak, return immidiately.  */
+      /* If the block was already a potential leak, return immediately.  */
       if ((block->garbage_t & POINT_MASK) == POINT_MAYBE)
         return;
       block->garbage_t = POINT_MAYBE | (block->garbage_t & INUSE_MASK);
     }
-  search_in_block (block, sure);	/* recursive.  */
+  search_in_block (block, ptr_is_sure);	/* recursive.  */
 }
 
-
 /* Fill all_busy_blocks.  */
 static void
 init_search (void)
@@ -140,15 +165,18 @@ init_search (void)
   struct malloc_header *blk;
   unsigned int nbr_busy_blocks;
   unsigned int i;
-  
+
+  /* For each heap.  */
   for (mdp = _firstmdesc; mdp != 0; mdp = mdp->info.inmem.next_mdesc)
     {
+      /* Count number of busy blocks.  */
       nbr_busy_blocks = 0;
       for (blk = mdp->_firstblock; blk; blk = blk->next)
 	{
 	  if (blk->state == MDBUSY)
 	    nbr_busy_blocks++;
 	}
+      /* Allocate and fill the array.  */
       mdp->nbr_busy_blocks = nbr_busy_blocks;
       if (nbr_busy_blocks == 0)
 	{
@@ -193,17 +221,15 @@ init_detector (void)
     chkr_printf ("Initialisation of detector...\n");
   
   for (mdp = _firstmdesc; mdp != 0; mdp = mdp->info.inmem.next_mdesc)
-    {
-      for (tmp = mdp->_firstblock; tmp != NULL_HEADER; tmp = tmp->next)
-	{
-	  tmp->garbage_t = POINT_NOT | (tmp->garbage_t & INUSE_MASK);
-	  tmp->g_flag = G_ALONE;
-	}
-    }
+    for (tmp = mdp->_firstblock; tmp != NULL_HEADER; tmp = tmp->next)
+      {
+	tmp->garbage_t = POINT_NOT | (tmp->garbage_t & INUSE_MASK);
+	tmp->g_flag = G_ALONE;
+      }
 }
 
 /* POINT_LEAK becomes POINT_SEEN.
- * POINT_SEEN remainds POINT_SEEN.
+ * POINT_SEEN remains POINT_SEEN.
  */
 static void
 init_evol_detector (void)
@@ -214,13 +240,23 @@ init_evol_detector (void)
   for (mdp = _firstmdesc; mdp != 0; mdp = mdp->info.inmem.next_mdesc)
     for (tmp = mdp->_firstblock; tmp != NULL_HEADER; tmp = tmp->next)
       {
-        if ((tmp->garbage_t & POINT_MASK) == POINT_LEAK 
-            || (tmp->garbage_t & POINT_MASK) == POINT_SEEN)
-	  tmp->garbage_t = POINT_SEEN | (tmp->garbage_t & INUSE_MASK);
-        else
+	if (tmp->state != MDBUSY)
+	  continue;
+
+	switch (tmp->garbage_t & POINT_MASK)
 	  {
+	  case POINT_LEAK:
+	  case POINT_MAYBE:
+	  case POINT_SEEN:
+	    tmp->garbage_t = POINT_SEEN | (tmp->garbage_t & INUSE_MASK);
+	    break;
+	  case POINT_SURE:
+	  case POINT_NOT:
 	    tmp->garbage_t = POINT_NOT | (tmp->garbage_t & INUSE_MASK);
 	    tmp->g_flag = G_ALONE;
+	    break;
+	  default:
+	    chkr_abort ();
 	  }
       }
 }
@@ -354,11 +390,9 @@ comp_history (struct malloc_header *bl1, struct malloc_header *bl2)
   /* Point to the tops of the history.  */
   ptr1 = (PTR *) header_to_history (bl1);
   ptr2 = (PTR *) header_to_history (bl2);
-  while (*ptr1 && *ptr2 && *ptr1++ == *ptr2++) ;
-  if (*ptr1 == *ptr2)
-    return 1;
-  else
-    return 0;
+  while (*ptr1 && *ptr2 && *ptr1++ == *ptr2++)
+    ;
+  return *ptr1 == *ptr2;
 }
 
 /* Gather the brothers.
@@ -381,7 +415,8 @@ regroup_brother_for_leaks (struct malloc_header *zone)
       if (block->state != MDBUSY)
 	continue;
 	
-      if ((block->garbage_t & POINT_MASK) == POINT_NOT 
+      if (((block->garbage_t & POINT_MASK) == POINT_NOT 
+	   || (block->garbage_t & POINT_MASK) == POINT_MAYBE )
           && block->g_flag == G_ALONE
 	  && comp_history (zone, block))
 	{
@@ -404,6 +439,9 @@ regroup_stack_for_leaks (void)
 {
   struct mdesc *mdp;
   struct malloc_header *block;
+
+  if (__chkr_verbose_detector)
+    chkr_printf ("Grouping leaks\n");
   
   /* Search all block.  */
   for (mdp = _firstmdesc; mdp != 0; mdp = mdp->info.inmem.next_mdesc)
@@ -461,6 +499,9 @@ disp_leaks (void)
   else
     chkr_printf (M_NBR_LEAKS, nbr_leaks, nbr_pot_leaks);
 
+  if (__chkr_verbose_detector)
+    chkr_printf ("Sorting leaks\n");
+
   /* Store the pointers to the leaks.  */
   pleaks = leaks = 
     (struct malloc_header **) alloca (nbr_leaks * sizeof (void *));
@@ -543,10 +584,38 @@ __chkr_evol_detector (void)
 void
 __chkr_clear_leaks (void)
 {
+  struct mdesc *mdp;
+  struct malloc_header *block;
+
   if (!__malloc_initialized)
     return;
-  init_evol_detector ();
+  init_detector ();
   all_search ();
+  
+  /* Search all block.  */
+  for (mdp = _firstmdesc; mdp != 0; mdp = mdp->info.inmem.next_mdesc)
+    for (block = mdp->_firstblock; block != NULL_HEADER; block = block->next)
+      {
+        /* Only busy blocks can be leaks.  */
+        if (block->state != MDBUSY)
+	  continue;
+	switch (block->garbage_t & POINT_MASK)
+	  {
+	  case POINT_NOT:
+	  case POINT_MAYBE:
+	    /* POINT_NOT is not reported as a leak.  */
+	    /* POINT_SEEN doesn't change.  */
+	    block->garbage_t = POINT_SEEN | (block->garbage_t & INUSE_MASK);
+	    break;
+	  case POINT_SURE:
+	    block->g_flag = G_ALONE;
+	    break;
+	  case POINT_SEEN:
+	  case POINT_LEAK:
+	  default:
+	    chkr_abort ();
+	  }
+      }
 }
 
 /* Gather the brothers.
