@@ -677,7 +677,7 @@ __chkr_dump_mheap (struct mdesc *mdp)
 #else
   while (tmp != NULL)
     {
-      chkr_printf ("%p (start: %p): ", tmp, (char*)tmp + HEADER_SIZE);
+      chkr_printf ("%p (start: %p): ", tmp, header_to_block (tmp));
       switch (tmp->state)
 	{
 	case MDBUSY:
