@@ -207,7 +207,13 @@ check_printf_format (char const *name, char const *fmt, va_list ap, int type, in
 	    }
 
 	  if (prec < 0)
-	    prec = -1;
+	    {
+	      chkr_report (M_C_FUN_LB_ET);
+	      chkr_printf ("%s: negative precision %d in format string\n", prec);
+	      chkr_disp_call_chain ();
+
+	      prec = -1;
+	    }
 	}
 
       if (*fmt == 'h')
@@ -460,8 +466,13 @@ check_printf_format (char const *name, char const *fmt, va_list ap, int type, in
 	      {
 		if (prec != -1)
 		  {
-		    w = strncharlen (*addr, prec);
-		    stubs_chkr_check_addr (*addr, strnsize (*addr, prec), CHKR_RO, "%s");
+		    if (prec > 0)
+		      {
+			w = strncharlen (*addr, prec);
+			stubs_chkr_check_addr (*addr, strnsize (*addr, prec), CHKR_RO, "%s");
+		      }
+		    else 
+		      w = 0;
 		  }
 		else
 		  {
