@@ -20,9 +20,16 @@
 #include "available-stubs.h"
 
 #include <libpq-fe.h>
-#include <libpq-int.h>
 #include "checker_api.h"
 
+#if 0
+/* Broken in Debian woody. */
+#include <libpq-int.h>
+#else
+struct pg_conn { int x; };
+struct pg_result { int x; };
+#endif
+
 #define CHKR_CHECK_STREAM(s) \
   stubs_chkr_check_addr (s, sizeof (FILE), CHKR_TW, "stream")
 
@@ -961,16 +968,16 @@ chkr_stub_PQprintTuples (PGresult * arg0, FILE * arg1, int arg2, int arg3, int a
 
 #ifdef HAVE_PQmblen
 /* From `/usr/include/postgresql/libpq-fe.h:287'.  */
-int chkr_stub_PQmblen (unsigned char * arg0) __asm__ (CHKR_PREFIX ("PQmblen"));
+int chkr_stub_PQmblen (unsigned char * arg0, int arg1) __asm__ (CHKR_PREFIX ("PQmblen"));
 int
-chkr_stub_PQmblen (unsigned char * arg0)
+chkr_stub_PQmblen (unsigned char * arg0, int arg1)
 {
   /* This function requires a stub */
   stubs_chkr_check_str (arg0, CHKR_RO, "string");
 #if USE_BI_JUMP
   __builtin_jump (PQmblen);
 #else
-  return PQmblen (arg0);
+  return PQmblen (arg0, arg1);
 #endif /* !USE_BI_JUMP */
 }
 #endif /* HAVE_PQmblen */
