@@ -1285,28 +1285,6 @@ chkr_stub_subwin (WINDOW *win, int nlines, int ncols, int begy, int begx)
 }
 #endif /* HAVE_subwin */
 
-#ifdef HAVE_tgoto
-/* From `/usr/include/ncurses/ncurses.h:213'.  */
-char * chkr_stub_tgoto (char * arg0, int arg1, int arg2)
-  __asm__ (CHKR_PREFIX ("tgoto"));
-char *
-chkr_stub_tgoto (char * arg0, int arg1, int arg2)
-{
-  /* This function requires a stub */
-  stubs_chkr_check_addr (arg0, sizeof (char), CHKR_XX);
-#if USE_BI_JUMP
-  __builtin_jump (tgoto);
-#else
-  return tgoto (arg0, arg1, arg2);
-  {
-    char * res;
-    res = tgoto (arg0, arg1, arg2);
-    return res;
-  }
-#endif /* !USE_BI_JUMP */
-}
-#endif /* HAVE_tgoto */
-
 #ifdef HAVE_tparm
 /* From `/usr/include/ncurses/ncurses.h:214'.  */
 char * chkr_stub_tparm (char * arg0, ... ) __asm__ (CHKR_PREFIX ("tparm"));
@@ -1329,28 +1307,6 @@ chkr_stub_tparm (char * arg0, ... )
 }
 #endif /* HAVE_tparm */
 
-#ifdef HAVE_tputs
-/* From `/usr/include/ncurses/ncurses.h:215'.  */
-int chkr_stub_tputs (char * arg0, int arg1, int (*) (char) arg2)
-  __asm__ (CHKR_PREFIX ("tputs"));
-int
-chkr_stub_tputs (char * arg0, int arg1, int (*) (char) arg2)
-{
-  /* This function requires a stub */
-  stubs_chkr_check_addr (arg0, sizeof (char), CHKR_XX);
-#if USE_BI_JUMP
-  __builtin_jump (tputs);
-#else
-  return tputs (arg0, arg1, arg2);
-  {
-    int res;
-    res = tputs (arg0, arg1, arg2);
-    return res;
-  }
-#endif /* !USE_BI_JUMP */
-}
-#endif /* HAVE_tputs */
-
 #ifdef HAVE_ungetch
 /* From `/usr/include/ncurses/ncurses.h:216'.  */
 int chkr_stub_ungetch (char arg0) __asm__ (CHKR_PREFIX ("ungetch"));
@@ -2202,4 +2158,8 @@ chkr_stub_slk_touch (void )
 }
 #endif /* HAVE_slk_touch */
 
+/* A little bit kludgy, but ncurses provide termcap compatibility.  */
+#undef HAVE_TERMCAP_H
+#include "stubs-termcap.c"
+
 #endif /* HAVE_NCURSES_H */
