@@ -160,7 +160,8 @@ int chkr_stub_fflush (FILE *stream) __asm__ (CHKR_PREFIX ("fflush"));
 int
 chkr_stub_fflush (FILE *stream)
 {
-  CHKR_CHECK_STREAM (stream);
+  if (stream != NULL)
+    CHKR_CHECK_STREAM (stream);
 #if USE_BI_JUMP
   __builtin_jump (fflush);
 #else
