@@ -170,15 +170,16 @@ void * chkr_stub_memchr (const void *buf, int c, size_t len)
 void *
 chkr_stub_memchr (const void *buf, int c, size_t len)
 {
-  void * res;
-  if (len == 0)
-    return NULL;
-  res = memchr (buf, c, len);
-  if (res)
-    stubs_chkr_check_addr (buf, res - buf + 1, CHKR_RO, "buf");
-  else
-    stubs_chkr_check_addr (buf, len, CHKR_RO, "buf");
-  return res;
+  while (len > 0)
+    {
+      stubs_chkr_check_addr (buf, 1, CHKR_RO, "buf");
+      if (*(unsigned char *)buf == c)
+	return buf;
+      len--;
+      buf++;
+    }
+
+  return NULL;
 }
 #endif /* HAVE_memchr */
 
@@ -376,15 +377,25 @@ chkr_stub_strrchr (const char *s, int c)
 size_t chkr_stub_strcspn (const char *str1, const char *str2)
   __asm__ (CHKR_PREFIX ("strcspn"));
 size_t
-chkr_stub_strcspn (const char *str1, const char *str2)
+chkr_stub_strcspn (const char *s, const char *reject)
 {
-  stubs_chkr_check_str (str1, CHKR_RO, "str1");
-  stubs_chkr_check_str (str2, CHKR_RO, "str2");
-#if USE_BI_JUMP
-  __builtin_jump (strcspn);
-#else
-  return strcspn (str1, str2);
-#endif /* !USE_BI_JUMP */
+  size_t count = 0;
+
+  stubs_chkr_check_str (reject, CHKR_RO, "reject");
+  for (;;)
+    {
+      stubs_chkr_check_addr (s, 1, CHKR_RO, "s");
+      
+      if (*s == '\0')
+	break;
+      
+      if (strchr (reject, *s++) == NULL)
+	++count;
+      else
+	return count;
+    }
+  
+  return count;
 }
 #endif /* HAVE_strcspn */
 
@@ -392,15 +403,28 @@ chkr_stub_strcspn (const char *str1, const char *str2)
 size_t chkr_stub_strspn (const char *str1, const char *str2)
   __asm__ (CHKR_PREFIX ("strspn"));
 size_t
-chkr_stub_strspn (const char *str1, const char *str2)
+chkr_stub_strspn (const char *s, const char *accept)
 {
-  stubs_chkr_check_str (str1, CHKR_RO, "str1");
-  stubs_chkr_check_str (str2, CHKR_RO, "str2");
-#if USE_BI_JUMP
-  __builtin_jump (strspn);
-#else
-  return strspn (str1, str2);
-#endif /* !USE_BI_JUMP */
+  const char *p;
+  const char *a;
+  size_t count = 0;
+
+  stubs_chkr_check_str (accept, CHKR_RO, "accept");
+  for (p = s; ; ++p)
+    {
+      stubs_chkr_check_addr (p, 1, CHKR_RO, "s");
+      if (*p == '\0')
+	break;
+      for (a = accept; *a != '\0'; ++a)
+	if (*p == *a)
+	  break;
+      if (*a == '\0')
+	return count;
+      else
+	++count;
+    }
+
+  return count;
 }
 #endif /* HAVE_strspn */
 
@@ -408,15 +432,25 @@ chkr_stub_strspn (const char *str1, const char *str2)
 char * chkr_stub_strpbrk (const char *str1, const char *str2)
   __asm__ (CHKR_PREFIX ("strpbrk"));
 char *
-chkr_stub_strpbrk (const char *str1, const char *str2)
+chkr_stub_strpbrk (const char *s, const char *accept)
 {
-  stubs_chkr_check_str (str1, CHKR_RO, "str1");
-  stubs_chkr_check_str (str2, CHKR_RO, "str2");
-#if USE_BI_JUMP
-  __builtin_jump (strpbrk);
-#else
-  return strpbrk (str1, str2);
-#endif /* !USE_BI_JUMP */
+  stubs_chkr_check_str (accept, CHKR_RO, "accept");
+
+  for (;;)
+    {
+      const char *a = accept;
+
+      stubs_chkr_check_addr (s, 1, CHKR_RO, "s");
+      if (*s == '\0')
+	break;
+
+      while (*a != '\0')
+	if (*a++ == *s)
+	  return (char *) s;
+      ++s;
+    }
+
+  return NULL;
 }
 #endif /* HAVE_strpbrk */
 
@@ -647,23 +681,14 @@ chkr_stub_strcasecmp (const char *s1, const char *s2)
 #endif /* HAVE_strcasecmp */
 
 #ifdef HAVE_strsep
-char * chkr_stub_strsep (char ** arg0, const char * arg1)
-  __asm__ (CHKR_PREFIX ("strsep"));
+char *chkr_stub_strsep (char **stringp, const char *delim)
+     __asm__ (CHKR_PREFIX ("strsep"));
 char *
-chkr_stub_strsep (char ** arg0, const char * arg1)
+chkr_stub_strsep (char **stringp, const char *delim)
 {
-  /* This function require a stub */
-  stubs_chkr_check_addr (arg0, sizeof (char *), CHKR_XX);
-  stubs_chkr_check_addr (arg1, sizeof (char), CHKR_XX);
-#if USE_BI_JUMP
-  __builtin_jump (strsep);
-#else
-  {
-    char * res;
-    res = strsep (arg0, arg1);
-    return res;
-  }
-#endif /* !USE_BI_JUMP */
+  stubs_chkr_check_addr (stringp, sizeof (char *), CHKR_RW, "stringp");
+  stubs_chkr_check_str (delim, CHKR_RO, "delim");
+  return strsep (stringp, delim);
 }
 #endif /* HAVE_strsep */
 
