@@ -39,7 +39,6 @@ Boston, MA 02111-1307, USA.
 #undef HAVE_swapon
 #undef HAVE_brk
 #undef HAVE_sbrk
-#undef HAVE_mkstemp
 #undef HAVE_profil
 #undef HAVE__xustat
 #undef HAVE_swapoff
@@ -1423,21 +1422,17 @@ chkr_stub_mktemp (char *file)
 #endif /* HAVE_mktemp */
 
 #ifdef HAVE_mkstemp
-int chkr_stub_mkstemp (char * arg0) __asm__ (CHKR_PREFIX ("mkstemp"));
+int chkr_stub_mkstemp (char *template) __asm__ (CHKR_PREFIX ("mkstemp"));
 int
-chkr_stub_mkstemp (char * arg0)
+chkr_stub_mkstemp (char *template)
 {
-  /* This function requires a stub */
-  stubs_chkr_check_addr (arg0, sizeof (char), CHKR_XX);
-#if USE_BI_JUMP
-  __builtin_jump (mkstemp);
-#else
-  {
-    int res;
-    res = mkstemp (arg0);
-    return res;
-  }
-#endif /* !USE_BI_JUMP */
+  int res;
+
+  stubs_chkr_check_str (template, CHKR_RW, "template");
+  res = mkstemp (template);
+  if (res >= 0)
+    fd_returned_by_system (res);
+  return res;
 }
 #endif /* HAVE_mkstemp */
 
@@ -1658,21 +1653,17 @@ chkr_stub_lockf (int fd, int func, off_t size)
 #endif /* HAVE_lockf */
 
 #ifdef HAVE_basename
-char * chkr_stub_basename (const char * arg0)
-  __asm__ (CHKR_PREFIX ("basename"));
+char * chkr_stub_basename (const char *filename)
+     __asm__ (CHKR_PREFIX ("basename"));
 char *
-chkr_stub_basename (const char * arg0)
+chkr_stub_basename (const char *filename)
 {
   /* This function requires a stub */
-  stubs_chkr_check_addr (arg0, sizeof (char), CHKR_RO, "buf");
+  stubs_chkr_check_str (filename, CHKR_RO, "filename");
 #if USE_BI_JUMP
   __builtin_jump (basename);
 #else
-  {
-    char * res;
-    res = basename (arg0);
-    return res;
-  }
+  return basename (filename);
 #endif /* !USE_BI_JUMP */
 }
 #endif /* HAVE_basename */
