@@ -1,4 +1,4 @@
-/* General purpose program for Checker
+/* General purpose driver for Checker
    Copyright 1994, 1995 Tristan Gingold
 		  Written July 1994 by Tristan Gingold
 
@@ -70,7 +70,12 @@ extern char *target;
 #ifndef GXX_PATH
 #define GXX_PATH (char *)0
 #endif
+
+/* Where g77 can be found.  */
+#ifndef G77_PATH
+#define G77_PATH (char *)0
 #endif
+#endif /* NEED_GCC */
 
 /* Where the libraries of Checker are.  */
 #ifndef CHECKER_LIB_PATH
@@ -104,7 +109,7 @@ fatal (const char *format, ...)
   exit (EXIT_FAILURE);
 }
 
-/* Allocate memory or failed.  */
+/* Allocate memory or fail.  */
 void *
 xmalloc (size_t size)
 {
@@ -115,7 +120,7 @@ xmalloc (size_t size)
   return res;
 }
 
-/* Allocate memory or failed.  */
+/* Reallocate memory or fail.  */
 void *
 xrealloc (void *ptr, size_t size)
 {
@@ -134,8 +139,8 @@ disp_version (int argc, char *argv[])
 }
 
 /* Replace all occurences of `@' with the real directory.  */
-char *
-expand_filename (char *file)
+const char *
+expand_filename (const char *file)
 {
   char *pos;
   static char *lib_path;
@@ -181,6 +186,10 @@ disp_path (int argc, char *argv[])
     printf ("gcc: %s\n", GCC_PATH);
   else
     printf ("gcc is the first gcc found in your path\n");
+  if (G77_PATH)
+    printf ("g77: %s\n", G77_PATH);
+  else
+    printf ("g77 is the first g77 found in your path\n");
   printf ("default checkerlib: %s\n", CHECKER_LIB_PATH);
   printf ("current checkerlib: %s\n", expand_filename ("@"));
 }
@@ -208,6 +217,9 @@ help (int argc, char *argv[])
   puts ("checker g++ [G++_ARGUMENTS]...");
   puts ("checker -g++ [G++_ARGUMENTS]...");
   puts ("  compile with GNU G++ for Checker.\n");
+  puts ("checker g77 [G77_ARGUMENTS]...");
+  puts ("checker -g77 [G77_ARGUMENTS]...");
+  puts ("  compile with GNU G77 for Checker.\n");
   puts ("checker -l");
   puts ("checker --libs");
   puts ("  disp known libraries.\n");
@@ -431,6 +443,12 @@ interpret (int argc, char *argv[])
 }
 #endif /* HAVE_BFD_H */
 
+const char * always_added_args[] = {
+#if defined (__GNU_LIBRARY__) || defined (__GLIBC__)
+  "-D__NO_MATH_INLINES",
+#endif
+  0
+};
 
 #ifdef NEED_GCC
 /* When called with gcc, checker is a wrapper like g++.  It inserts args at
@@ -440,13 +458,13 @@ interpret (int argc, char *argv[])
    environment variable CHECKER_LIB_DIR.  This is mainly for testing
    purposes.  */
 /* Args to insert, for gcc and g++.  */
-char *added_args_for_gcc[] = { "-B@/", 0};
-char *added_args_for_gpp[] = { "-B@/", 0};
-char *added_libs_for_gcc[] = { 0};
-char *added_libs_for_gxx[] = { 0};
+const char * added_args_for_gcc[] = { "-B@/", 0};
+const char * added_args_for_gxx[] = { "-B@/", 0};
+const char * added_libs_for_gcc[] = { 0};
+const char * added_libs_for_gxx[] = { 0};
 #else
+const char * added_args[] = {
 #ifdef GCCCHECKER
-char *added_args_for_gcc[] = { 
 	"-fcheck-memory-usage", "-fprefix-function-name",
 #ifdef NEED_CHECKER_SPECS_FOR_GCC
 	"-specs=@/specs",
@@ -457,26 +475,19 @@ char *added_args_for_gcc[] = {
 	"-D__CHECKER__",
 	"-DMALLOC_0_RETURNS_NULL",
 	"-L@", 0};
-char *added_args_for_gxx[] = { 
-	"-fcheck-memory-usage", "-fprefix-function-name",
-#ifdef NEED_CHECKER_SPECS_FOR_GCC
-	"-specs=@/specs",
-	"-checker",
-#endif
-	"-g",
-	"-I@/include",
-	"-D__CHECKER__",
-	"-DMALLOC_0_RETURNS_NULL",
-	"-L@", 0};
-char *added_libs_for_gcc[] = { 0};
-char *added_libs_for_gxx[] = { 0};
+const char * *added_args_for_gcc = added_args;
+const char * *added_args_for_gxx = added_args;
+const char * *added_args_for_g77 = added_args;
+const char * added_libs_for_gcc[] = { 0};
+const char * added_libs_for_gxx[] = { 0};
+const char * added_libs_for_g77[] = { 0};
 #endif
 #endif
 
 
 /* Replace all occurences of `@' with the real directory.  */
 void
-expand_path (char **tab)
+expand_path (const char **tab)
 {
   for (; *tab; tab++)
     *tab = expand_filename (*tab);
@@ -495,7 +506,7 @@ void
 read_available_stubs (void)
 {
   int len;
-  char *path;
+  const char *path;
   char *filename;
   DIR *dirp;
   struct dirent *direntp;
@@ -544,6 +555,7 @@ read_available_stubs (void)
   closedir (dirp);
 }
 
+/* Display known libraries.  */
 void
 disp_libs (int argc, char *argv[])
 {
@@ -556,9 +568,11 @@ disp_libs (int argc, char *argv[])
   printf ("\n");
 }
 
+/*  Array of argument for compiling.  */
 char **compiler_args;
 int nbr_compiler_args;
 
+/* Split COMPILER_NAME into an argument list COMPILER_ARGS.  */
 void
 set_compiler (const char *compiler_name)
 {
@@ -589,15 +603,28 @@ set_compiler (const char *compiler_name)
   free (compiler);
 }
 
+/* Flags for COMPILE.  */
+/* Add libstdc++ before stubs.  This is needed since libstc++ call checkered
+   version of libc.  */
+#define FLAG_STDCXX	(1 << 0)
+
 /* Compile with gcc.  */
+/* Additionnal arguments are inserted.
+   If linked is performed, additionnal libraries can be added.
+   The tricky part is stubs.
+   All -lLIBRARY argument are scaned.  If a stub exists for LIBRARY, then the
+   argument is replaced by -lchkr_LIBRARY, and -lLIBRARY is added inside the
+   stubs command line part.  */
 void
-compile (char *added_args[], char *added_libs[], char *prog, char *prog1,
+compile (const char *added_args[], const char *added_libs[],
+	 unsigned int flags,
+	 char *prog, char *prog1,
 	 int argc, char *argv[])
 {
   int i,j;
 
   /* The new argument array.  */
-  char **newargv;
+  const char **newargv;
 
   /* The libraries with stubs.  */
   char **libs;
@@ -607,6 +634,7 @@ compile (char *added_args[], char *added_libs[], char *prog, char *prog1,
   int library = 1;
 
   /* Number of items in added_{args, libs}.  */
+  int nbr_always_args;
   int nbr_added_args;
   int nbr_added_libs;
   int nargc;
@@ -617,6 +645,10 @@ compile (char *added_args[], char *added_libs[], char *prog, char *prog1,
     set_compiler (prog1);
 
   /* Compute the number of items.  */
+  for (i = 0; always_added_args[i]; i++)
+    ;
+  nbr_always_args = i;
+
   for (i = 0; added_args[i]; i++)
     ;
   nbr_added_args = i;
@@ -626,8 +658,13 @@ compile (char *added_args[], char *added_libs[], char *prog, char *prog1,
   nbr_added_libs = i;
 
   /* Alloc space for the array.  */
-  newargv = (char **) alloca ((nbr_compiler_args + 2 * argc + nbr_added_args
-			       + 2 * nbr_added_libs) * sizeof (char*));
+  newargv = (const char **) alloca 
+    ((4 + nbr_compiler_args
+      + 2 * argc
+      + nbr_always_args
+      + nbr_added_args
+      + 2 * nbr_added_libs)
+     * sizeof (char *));
   libs = (char **) alloca (argc * sizeof (char *));
   nlib = 0;
 
@@ -635,8 +672,8 @@ compile (char *added_args[], char *added_libs[], char *prog, char *prog1,
   for (i = 0; i < argc; i++)
     if (argv[i][0] == '-')
       {
-        /* The option -nostdlib, -c, -S, -E, -M and -MM prevent from linking.
-           Note also that 'gcc -v' prevent also from linking.  */
+        /* The options -nostdlib, -c, -S, -E, -M and -MM prevent from linking.
+           Note also that 'gcc -v' also prevents from linking.  */
         if (strcmp (argv[i], "nostdlib") == 0)
           library = 0;
         else if (argv[i][1] && argv[i][2] == 0)
@@ -692,6 +729,8 @@ compile (char *added_args[], char *added_libs[], char *prog, char *prog1,
   nargc = nbr_compiler_args;
   
   /* Insert the (added) args.  */
+  for (i = 0; i < nbr_always_args; i++)
+    newargv[nargc++] = always_added_args[i];
   for (i = 0; i < nbr_added_args; i++)
     newargv[nargc++] = added_args[i];
   
@@ -705,6 +744,9 @@ compile (char *added_args[], char *added_libs[], char *prog, char *prog1,
       int len;
       char *p;
 
+      if (flags & FLAG_STDCXX)
+	newargv[nargc++] = "-lstdc++";
+
       /* The checker library.  */
       newargv[nargc++] = expand_filename ("@/gccchecker.o");
       newargv[nargc++] = expand_filename ("@/begin-stubs.o");
@@ -753,7 +795,7 @@ compile (char *added_args[], char *added_libs[], char *prog, char *prog1,
     }
    
   /* Compile now.  */
-  execvp (newargv[0], newargv);
+  execvp (newargv[0], (char * const *) newargv);
   perror ("gcc not found");
   exit (1);
 }
@@ -764,9 +806,8 @@ gcc (int argc, char *argv[])
 {
   expand_path (added_args_for_gcc);
   expand_path (added_libs_for_gcc);
-  compile (added_args_for_gcc, added_libs_for_gcc,
-  	   GCC_PATH,
-	   "gcc", argc, argv);
+  compile (added_args_for_gcc, added_libs_for_gcc, 0,
+	   GCC_PATH, "gcc", argc, argv);
 }
 
 /* Compile with g++.  */
@@ -775,9 +816,18 @@ gplusplus (int argc, char *argv[])
 {
   expand_path (added_args_for_gxx);
   expand_path (added_libs_for_gxx);
-  compile (added_args_for_gxx, added_libs_for_gxx,
-  	   GXX_PATH,
-	   "g++", argc, argv);
+  compile (added_args_for_gxx, added_libs_for_gxx, FLAG_STDCXX,
+  	   GXX_PATH, "g++", argc, argv);
+}
+
+/* Compile with g77.  */
+void
+g77 (int argc, char *argv[])
+{
+  expand_path (added_args_for_g77);
+  expand_path (added_libs_for_g77);
+  compile (added_args_for_g77, added_libs_for_g77, 0,
+  	   G77_PATH, "g77", argc, argv);
 }
 
 /* Compile with specified compiler.  */
@@ -786,8 +836,8 @@ gen_compile (int argc, char *argv[])
 {
   expand_path (added_args_for_gcc);
   expand_path (added_libs_for_gcc);
-  compile (added_args_for_gcc, added_libs_for_gcc, NULL,
-  	   argv[0] + sizeof ("--compile=") - 1, argc, argv);
+  compile (added_args_for_gcc, added_libs_for_gcc, 0,
+	   NULL, argv[0] + sizeof ("--compile=") - 1, argc, argv);
 }
 
 #endif /* NEED_GCC */
@@ -837,9 +887,9 @@ struct flags
 };
 
 struct flags commands[] = {
-  { 0, 'h', "--help",	&help},
+  { 0, 'h', "--help",		&help},
   { 0, 'v', "--version",	&disp_version},
-  { 0, 'p', "--path",	&disp_path},
+  { 0, 'p', "--path",		&disp_path},
 #ifdef HAVE_BFD_H
   { 0, 'i', "--interpret",	&interpret},
 #endif
@@ -848,6 +898,8 @@ struct flags commands[] = {
   { 0, 0,  "-gcc",	&gcc},
   { 0, 0,  "g++",	&gplusplus},
   { 0, 0,  "-g++",	&gplusplus},
+  { 0, 0,  "g77",	&g77},
+  { 0, 0,  "-g77",	&g77},
   { 0, 'l', "--libs",	&disp_libs},
   { 1, 0,   "--compile",&gen_compile},
 #endif
@@ -869,11 +921,15 @@ main (int argc, char *argv[])
     basename--;
 
 #ifdef NEED_GCC
+  /* Special behavior according to argv[0].  */
   if (strcmp ("checkergcc", basename) == 0)
     gcc (argc, argv);
  
   if (strcmp ("checkerg++", basename) == 0)
     gplusplus (argc, argv);
+
+  if (strcmp ("checkerg77", basename) == 0)
+    g77 (argc, argv);
 #endif /* NEED_GCC */
    
   if (argc < 2)
